@@ -1,7 +1,7 @@
 package Carmel;
 use strict;
 use 5.010_001;
-use version; our $VERSION = version->declare('v0.1.5');
+use version; our $VERSION = version->declare('v0.1.6');
 
 1;
 __END__
@@ -32,6 +32,9 @@ Carmel - CPAN Artifact Repository Manager
   # Runs your perl script with modules from artifacts
   carmel exec perl ...
 
+  # See which INC entry is added
+  carmel exec perl -V
+
   # Runs your perl script with a checker to guarantee everything is loaded from Carmel
   carmel exec perl -MDevel::Carmel script.pl
 
@@ -58,10 +61,10 @@ paths to include upon runtime.
 =head1 HOW IT WORKS
 
 Carmel will keep the build directory (artifacts) after a cpanm
-installation in a repository, which defaults to C<$HOME/.perl-carmel/builds/{version}-{archname}>,
+installation in a repository, which defaults to C<$HOME/.perl-carmel/{version}-{archname}/builds>,
 and your directory structure would look like:
 
-  $HOME/.perl-carmel/builds/5.20.1-darwin-2level
+  $HOME/.perl-carmel/5.20.1-darwin-2level/builds
     Plack-1.0033/
       blib/
         arch/
