@@ -11,6 +11,9 @@ use Module::CoreList;
 use Module::CPANfile;
 use Path::Tiny ();
 use Pod::Usage ();
+use Try::Tiny;
+
+our $UseSystem = 0; # unit testing
 
 sub new {
     my $class = shift;
@@ -26,18 +29,29 @@ sub run {
     my $call = $self->can("cmd_$cmd")
       or die "Could not find command '$cmd'";
 
-    $self->$call(@args);
+    try {
+        $self->$call(@args);
+    } catch {
+        warn $_;
+        return 255;
+    };
+
+    return 0;
 }
 
-sub base_dir {
-    # It just needs to be a temporary location to make re-installation faster
+sub repository_base {
     my $self = shift;
-    Path::Tiny->new("$ENV{HOME}/.perl-carmel/cache/$self->{perl_arch}");
+    Path::Tiny->new($ENV{PERL_CARMEL_REPO} || "$ENV{HOME}/.perl-carmel/$self->{perl_arch}");
 }
 
-sub repository_path {
+sub cache_dir {
     my $self = shift;
-    Path::Tiny->new($ENV{PERL_CARMEL_REPO} || "$ENV{HOME}/.perl-carmel/builds/$self->{perl_arch}");
+    $self->repository_base->child("cache");
+}
+
+sub build_dir {
+    my $self = shift;
+    $self->repository_base->child("builds");
 }
 
 sub repo {
@@ -47,7 +61,7 @@ sub repo {
 
 sub build_repo {
     my $self = shift;
-    Carmel::Repository->new($self->repository_path);
+    Carmel::Repository->new($self->build_dir);
 }
 
 sub cmd_help {
@@ -86,7 +100,6 @@ sub install_from_cpanfile {
         },
     );
 
-
     if (my @missing = $requirements->required_modules) {
         my $cpanfile = Module::CPANfile->from_prereqs({
             runtime => {
@@ -99,9 +112,10 @@ sub install_from_cpanfile {
     my @artifacts;
     $self->resolve(sub { push @artifacts, $_[0] });
 
+    # $self->requirements has been ugpraded at this point with the whole subreqs
     printf "---> Complete! %d cpanfile dependencies. %d modules installed.\n" .
       "---> Use `carmel show [module]` to see where a module is installed.\n",
-      scalar($self->requirements->required_modules), scalar(@artifacts);
+      scalar($self->build_requirements->required_modules), scalar(@artifacts);
 }
 
 sub is_core {
@@ -127,7 +141,7 @@ sub install {
     my $dir = Path::Tiny->tempdir;
     local $ENV{PERL_CPANM_HOME} = $dir;
     local $ENV{PERL_CPANM_OPT};
-    system $^X, "-S", "cpanm", "--quiet", "--notest", "-L", $self->base_dir, @args if @args;
+    system $^X, "-S", "cpanm", "--quiet", "--notest", "-L", $self->cache_dir, @args if @args;
 
     for my $ent ($dir->child("latest-build")->children) {
         next unless $ent->is_dir && $ent->child("blib/meta/install.json")->exists;
@@ -151,7 +165,7 @@ sub cmd_exec {
     my($self, @args) = @_;
     my %env = $self->env;
     %ENV = (%env, %ENV);
-    exec @args;
+    $UseSystem ? system(@args) : exec @args;
 }
 
 sub cmd_find {
@@ -219,43 +233,20 @@ sub cmd_index {
 sub write_index {
     my($self, $fh) = @_;
 
-    my @packages;
+    require Carton::Index;
+    require Carton::Package;
+
+    my $index = Carton::Index->new(generator => "Carmel $Carmel::VERSION");
+
     $self->resolve(sub {
         my $artifact = shift;
-        while (my($package, $data) = each %{$artifact->install->{provides}}) {
-            push @packages, {
-                package => $package,
-                version => $data->{version} || 'undef',
-                pathname => $artifact->install->{pathname},
-            }
+        while (my($pkg, $data) = each %{$artifact->install->{provides}}) {
+            my $package = Carton::Package->new($pkg, $data->{version} || 'undef', $artifact->install->{pathname});
+            $index->add_package($package);
         }
     });
 
-    print $fh <<EOF;
-File:         02packages.details.txt
-URL:          http://www.perl.com/CPAN/modules/02packages.details.txt
-Description:  Package names found in Carmel build artifacts
-Columns:      package name, version, path
-Intended-For: Automated fetch routines, namespace documentation.
-Written-By:   Carmel $Carmel::VERSION
-Line-Count:   @{[ $#packages + 1 ]}
-Last-Updated: @{[ scalar localtime ]}
-
-EOF
-
-    for my $p (sort { $a->{package} cmp $b->{package} } @packages) {
-        print $fh sprintf "%s %s  %s\n", pad($p->{package}, 32), pad($p->{version}, 10, 1), $p->{pathname};
-    }
-}
-
-sub pad {
-    my($str, $len, $left) = @_;
-
-    my $howmany = $len - length($str);
-    return $str if $howmany <= 0;
-
-    my $pad = " " x $howmany;
-    return $left ? "$pad$str" : "$str$pad";
+    $index->write($fh);
 }
 
 sub try_snapshot {
@@ -319,6 +310,8 @@ sub apply_snapshot_recursively {
 sub resolve_recursive {
     my($self, $root_reqs, $requirements, $seen, $cb, $missing_cb, $depth) = @_;
 
+    # TODO rather than mutating $root_reqs directly, we should create a new object
+    # that allows accesing the result $requirements
     for my $module (sort $requirements->required_modules) {
         next if $module eq 'perl';
 
